@@ -1,151 +1,133 @@
 <?php
 /* vim: set expandtab tabstop=4 shiftwidth=4: */
-// +--------------------------------------------------------------------------+
-// | Net_LDAP                                                                 |
-// +--------------------------------------------------------------------------+
-// | Copyright (c) 1997-2003 The PHP Group                                    |
-// +--------------------------------------------------------------------------+
-// | This library is free software; you can redistribute it and/or            |
-// | modify it under the terms of the GNU Lesser General Public               |
-// | License as published by the Free Software Foundation; either             |
-// | version 2.1 of the License, or (at your option) any later version.       |
-// |                                                                          |
-// | This library is distributed in the hope that it will be useful,          |
-// | but WITHOUT ANY WARRANTY; without even the implied warranty of           |
-// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        |
-// | Lesser General Public License for more details.                          |
-// |                                                                          |
-// | You should have received a copy of the GNU Lesser General Public         |
-// | License along with this library; if not, write to the Free Software      |
-// | Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA |
-// +--------------------------------------------------------------------------+
-// | Authors: Jan Wagner, Tarjej Huse                                         |
-// +--------------------------------------------------------------------------+
-//
-// $Id: Entry.php,v 1.36 2007/04/23 08:11:09 beni Exp $
 
-require_once("PEAR.php");
-require_once('Util.php');
+require_once 'PEAR.php';
+require_once 'Util.php';
 
 /**
- * Object representation of a directory entry
- *
- * This class represents a directory entry. You can add, delete, replace
- * attributes and their values, rename the entry, delete the entry.
- *
- * @package Net_LDAP
- * @author Jan Wagner <wagner@netsols.de>
- * @author Tarjej Huse
- * @version $Revision: 1.36 $
- */
+* Object representation of a directory entry
+*
+* This class represents a directory entry. You can add, delete, replace
+* attributes and their values, rename the entry, delete the entry.
+*
+* @category Net
+* @package  Net_LDAP
+* @author   Jan Wagner <wagner@netsols.de>
+* @author   Tarjej Huse <tarjei@bergfald.no>
+* @license  http://www.gnu.org/copyleft/lesser.html LGPL
+* @version  CVS: $Id: Entry.php,v 1.47 2007/10/24 06:29:32 beni Exp $
+* @link     http://pear.php.net/package/Net_LDAP/
 
+*/
 class Net_LDAP_Entry extends PEAR
 {
     /**
-     * Entry ressource identifier
-     *
-     * @access private
-     * @var ressourcee
-     */
+    * Entry ressource identifier
+    *
+    * @access private
+    * @var ressourcee
+    */
     var $_entry = null;
 
     /**
-     * LDAP ressource identifier
-     *
-     * @access private
-     * @var ressource
-     */
+    * LDAP ressource identifier
+    *
+    * @access private
+    * @var ressource
+    */
     var $_link = null;
 
     /**
-     * Net_LDAP object
-     *
-     * This object will be used for updating and schema checking
-     *
-     * @access private
-     * @var object Net_LDAP
-     */
+    * Net_LDAP object
+    *
+    * This object will be used for updating and schema checking
+    *
+    * @access private
+    * @var object Net_LDAP
+    */
     var $_ldap = null;
 
     /**
-     * Distinguished name of the entry
-     *
-     * @access private
-     * @var string
-     */
+    * Distinguished name of the entry
+    *
+    * @access private
+    * @var string
+    */
     var $_dn = null;
 
     /**
-     * Attributes
-     *
-     * @access private
-     * @var array
-     */
+    * Attributes
+    *
+    * @access private
+    * @var array
+    */
     var $_attributes = array();
 
     /**
-     * Original attributes before any modification
-     *
-     * @access private
-     * @var array
-     */
+    * Original attributes before any modification
+    *
+    * @access private
+    * @var array
+    */
     var $_original = array();
 
 
     /**
-     * Map of attribute names
-     *
-     * @access private
-     * @var array
-     */
+    * Map of attribute names
+    *
+    * @access private
+    * @var array
+    */
     var $_map = array();
 
 
     /**
-     * Is this a new entry?
-     *
-     * @access private
-     * @var boolean
-     */
+    * Is this a new entry?
+    *
+    * @access private
+    * @var boolean
+    */
     var $_new = true;
 
     /**
-     * New distinguished name
-     *
-     * @access private
-     * @var string
-     */
+    * New distinguished name
+    *
+    * @access private
+    * @var string
+    */
     var $_newdn = null;
 
     /**
-     * Shall the entry be deleted?
-     *
-     * @access private
-     * @var boolean
-     */
+    * Shall the entry be deleted?
+    *
+    * @access private
+    * @var boolean
+    */
     var $_delete = false;
 
     /**
-     * Map with changes to the entry
-     *
-     * @access private
-     * @var array
-     */
+    * Map with changes to the entry
+    *
+    * @access private
+    * @var array
+    */
     var $_changes = array("add"     => array(),
                           "delete"  => array(),
                           "replace" => array()
                           );
     /**
-     * Constructor
-     *
-     * Constructor of the entry. Sets up the distinguished name and the entries
-     * attributes.
-     *
-     * @access protected
-     * @param Net_LDAP|ressource|array $ldap   Net_LDAP object, ldap-link ressource or array of attributes
-     * @param string|ressource         $entry  Either a DN or a LDAP-Entry
-     * @return none
-     */
+    * Internal Constructor
+    *
+    * Constructor of the entry. Sets up the distinguished name and the entries
+    * attributes.
+    * You should not call this method manually! Use {@link Net_LDAP_Entry::createFresh()} instead!
+    *
+    * @param Net_LDAP|ressource|array &$ldap Net_LDAP object, ldap-link ressource or array of attributes
+    * @param string|ressource         $entry Either a DN or a LDAP-Entry ressource
+    *
+    * @access protected
+    * @return none
+    */
     function Net_LDAP_Entry(&$ldap, $entry = null)
     {
         $this->PEAR('Net_LDAP_Error');
@@ -162,30 +144,60 @@ class Net_LDAP_Entry extends PEAR
         } elseif (is_resource($ldap)) {
             $this->_link = $ldap;
         } elseif (is_array($ldap)) {
-            $this->_setAttributes($ldap);
+            $this->_setAttributes($ldap);  // setup attrs manually
         }
 
         if (is_resource($this->_entry) && is_resource($this->_link)) {
             $this->_new = false;
             $this->_dn  = @ldap_get_dn($this->_link, $this->_entry);
-            $this->_setAttributes();
+            $this->_setAttributes();  // fetch attributes from server
         }
     }
 
     /**
-     * Get or set the distinguished name of the entry
-     *
-     * If called without an argument the current (or the new DN if set) DN gets returned.
-     * If you provide an DN, this entry is moved to the new location specified if a DN existed.
-     * If the DN was not set, the DN gets initialized. Call {@link update()} to actually create
-     * the new Entry in the directory.
-     *
-     * Please note that special characters (eg german umlauts) should be encoded using utf8_encode().
-     *
-     * @access public
-     * @param string $dn New distinguished name
-     * @return string|true Distinguished name (or true if a new DN was provided)
-     */
+    * Creates a fresh entry that may be added to the directory later on
+    *
+    * The method should be called statically: $entry = Net_LDAP_Entry::createFresh();
+    *
+    * Use this method, if you want to initialize a fresh entry.
+    * The attributes parameter is as following:
+    * <code>
+    * $attrs = array( 'attribute1' => array('value1', 'value2'),
+    *                 'attribute2' => 'single value'
+    *          );
+    * </code>
+    *
+    * @param string $dn    DN of the Entry
+    * @param array  $attrs Attributes of the entry
+    *
+    * @static
+    * @return Net_LDAP_Entry
+    */
+    function createFresh($dn, $attrs = array())
+    {
+        if (!is_array($attrs)) {
+            return PEAR::raiseError("Unable to create fresh entry: Parameter \$attrs needs to be an array!");
+        }
+
+        $entry = new Net_LDAP_Entry($attrs, $dn);
+        return $entry;
+    }
+
+    /**
+    * Get or set the distinguished name of the entry
+    *
+    * If called without an argument the current (or the new DN if set) DN gets returned.
+    * If you provide an DN, this entry is moved to the new location specified if a DN existed.
+    * If the DN was not set, the DN gets initialized. Call {@link update()} to actually create
+    * the new Entry in the directory.
+    *
+    * Please note that special characters (eg german umlauts) should be encoded using utf8_encode().
+    *
+    * @param string $dn New distinguished name
+    *
+    * @access public
+    * @return string|true Distinguished name (or true if a new DN was provided)
+    */
     function dn($dn = null)
     {
         if (false == is_null($dn)) {
@@ -201,20 +213,26 @@ class Net_LDAP_Entry extends PEAR
     }
 
     /**
-     * Sets the internal attributes array
-     *
-     * This fetches the values for the attributes from the server.
-     * The attribute Syntax will be checked so binary attributes will be returned
-     * as binary values
-     *
-     * @access private
-     * @param array $attributes
-     */
+    * Sets the internal attributes array
+    *
+    * This fetches the values for the attributes from the server.
+    * The attribute Syntax will be checked so binary attributes will be returned
+    * as binary values.
+    *
+    * Attributes may be passed directly via the $attributes parameter to setup this
+    * entry manually. This overrides attribute fetching from the server.
+    *
+    * @param array $attributes Attributes to set for this entry
+    *
+    * @access private
+    * @return void
+    */
     function _setAttributes($attributes = null)
     {
-        // fetch attributes from the server
-        if (is_null($attributes) && is_resource($this->_entry) && is_resource($this->_link))
-        {
+        /*
+        * fetch attributes from the server
+        */
+        if (is_null($attributes) && is_resource($this->_entry) && is_resource($this->_link)) {
             // fetch schema
             if (is_a($this->_ldap, 'Net_LDAP')) {
                 $schema =& $this->_ldap->schema();
@@ -223,7 +241,7 @@ class Net_LDAP_Entry extends PEAR
             $attributes = array();
             do {
                 if (empty($attr)) {
-                    $ber = null;
+                    $ber  = null;
                     $attr = @ldap_first_attribute($this->_link, $this->_entry, $ber);
                 } else {
                     $attr = @ldap_next_attribute($this->_link, $this->_entry, $ber);
@@ -243,6 +261,9 @@ class Net_LDAP_Entry extends PEAR
             } while ($attr);
         }
 
+        /*
+        * set attribute data directly, if passed
+        */
         if (is_array($attributes) && count($attributes) > 0) {
             if (isset($attributes["count"]) && is_numeric($attributes["count"])) {
                 unset($attributes["count"]);
@@ -265,19 +286,21 @@ class Net_LDAP_Entry extends PEAR
                 $this->_attributes[$k] = $v;
             }
         }
+
         // save a copy for later use
         $this->_original = $this->_attributes;
     }
+
     /**
-     * Get the values of all attributes in a hash
-     *
-     * The returned hash has the form
-     * array('attributename' => 'single value',
-     *       'attributename' => array('value1', value2', value3'))
-     *
-     * @access public
-     * @return array Hash of all attributes with their values
-     */
+    * Get the values of all attributes in a hash
+    *
+    * The returned hash has the form
+    * <code>array('attributename' => 'single value',
+    *       'attributename' => array('value1', value2', value3'))</code>
+    *
+    * @access public
+    * @return array Hash of all attributes with their values
+    */
     function getValues()
     {
         $attrs = array();
@@ -288,22 +311,23 @@ class Net_LDAP_Entry extends PEAR
     }
 
     /**
-     * Get the value of a specific attribute
-     *
-     * The first parameter is the name of the attribute
-     * The second parameter influences the way the value is returned:
-     * 'single': only the first value is returned as string
-     * 'all': all values including the value count are returned in an
-     *               array
-     * 'default': in all other cases an attribute value with a single value is
-     *            returned as string, if it has multiple values it is returned
-     *            as an array (without value count)
-     *
-     * @access public
-     * @param string $attr Attribute name
-     * @param string $option Option
-     * @return string|array|PEAR_Error string, array or PEAR_Error
-     */
+    * Get the value of a specific attribute
+    *
+    * The first parameter is the name of the attribute
+    * The second parameter influences the way the value is returned:
+    * 'single': only the first value is returned as string
+    * 'all': all values including the value count are returned in an
+    *               array
+    * 'default': in all other cases an attribute value with a single value is
+    *            returned as string, if it has multiple values it is returned
+    *            as an array (without value count)
+    *
+    * @param string $attr   Attribute name
+    * @param string $option Option
+    *
+    * @access public
+    * @return string|array|PEAR_Error string, array or PEAR_Error
+    */
     function getValue($attr, $option = null)
     {
         $attr = $this->_getAttrName($attr);
@@ -322,10 +346,10 @@ class Net_LDAP_Entry extends PEAR
     }
 
     /**
-     * Alias function of getValue for perl-ldap interface
-     *
-     * @see getValue()
-     */
+    * Alias function of getValue for perl-ldap interface
+    *
+    * @see getValue()
+    */
     function get_value()
     {
         $args = func_get_args();
@@ -333,23 +357,24 @@ class Net_LDAP_Entry extends PEAR
     }
 
     /**
-     * Returns an array of attributes names
-     *
-     * @access public
-     * @return array Array of attribute names
-     */
+    * Returns an array of attributes names
+    *
+    * @access public
+    * @return array Array of attribute names
+    */
     function attributes()
     {
         return array_keys($this->_attributes);
     }
 
     /**
-     * Returns whether an attribute exists or not
-     *
-     * @access public
-     * @param string $attr Attribute name
-     * @return boolean
-     */
+    * Returns whether an attribute exists or not
+    *
+    * @param string $attr Attribute name
+    *
+    * @access public
+    * @return boolean
+    */
     function exists($attr)
     {
         $attr = $this->_getAttrName($attr);
@@ -357,23 +382,25 @@ class Net_LDAP_Entry extends PEAR
     }
 
     /**
-     * Adds a new attribute or a new value to an existing attribute
-     *
-     * The paramter has to be an array of the form:
-     * array('attributename' => 'single value',
-     *       'attributename' => array('value1', 'value2))
-     * When the attribute already exists the values will be added, else the
-     * attribute will be created. These changes are local to the entry and do
-     * not affect the entry on the server until update() is called.
-     *
-     * Note, that you can add values of attributes that you haven't selected, but if
-     * you do so, {@link getValue()} and {@link getValues()} will only return the
-     * values you added, _NOT_ all values present on the server. To avoid this, just refetch
-     * the entry after calling {@link update()} or select the attribute.
-     *
-     * @access public
-     * @param array $attr
-     */
+    * Adds a new attribute or a new value to an existing attribute
+    *
+    * The paramter has to be an array of the form:
+    * array('attributename' => 'single value',
+    *       'attributename' => array('value1', 'value2))
+    * When the attribute already exists the values will be added, else the
+    * attribute will be created. These changes are local to the entry and do
+    * not affect the entry on the server until update() is called.
+    *
+    * Note, that you can add values of attributes that you haven't selected, but if
+    * you do so, {@link getValue()} and {@link getValues()} will only return the
+    * values you added, _NOT_ all values present on the server. To avoid this, just refetch
+    * the entry after calling {@link update()} or select the attribute.
+    *
+    * @param array $attr Attributes to add
+    *
+    * @access public
+    * @return true|Net_LDAP_Error
+    */
     function add($attr = array())
     {
         if (false == is_array($attr)) {
@@ -394,7 +421,7 @@ class Net_LDAP_Entry extends PEAR
                 $this->_attributes[$k] = array_merge($this->_attributes[$k], $v);
             } else {
                 $this->_map[strtolower($k)] = $k;
-                $this->_attributes[$k] = $v;
+                $this->_attributes[$k]      = $v;
             }
             // save changes for update()
             if (empty($this->_changes["add"][$k])) {
@@ -402,36 +429,40 @@ class Net_LDAP_Entry extends PEAR
             }
             $this->_changes["add"][$k] = array_merge($this->_changes["add"][$k], $v);
         }
+        $return = true;
+        return $return;
     }
 
     /**
-     * Deletes an attribute or a value
-     *
-     * The parameter can be one of the following:
-     *
-     * "attributename" - The attribute as a whole will be deleted
-     * array("attributename1", "attributename2) - All given attributes will be
-     *                                            deleted
-     * array("attributename" => "value") - The value will be deleted
-     * array("attributename" => array("value1", "value2") - The given values
-     *                                                      will be deleted
-     * If $attr is null or omitted , then the whole Entry will be deleted!
-     *
-     * These changes are local to the entry and do
-     * not affect the entry on the server until {@link update()} is called.
-     *
-     * Please note that you must select the attribute (at $ldap->search() for example)
-     * to be able to delete values of it, Otherwise {@link update()} will silently fail
-     * and remove nothing.
-     *
-     * @access public
-     * @param string|array $attr
-     */
+    * Deletes an whole attribute or a value or the whole entry
+    *
+    * The parameter can be one of the following:
+    *
+    * "attributename" - The attribute as a whole will be deleted
+    * array("attributename1", "attributename2) - All given attributes will be
+    *                                            deleted
+    * array("attributename" => "value") - The value will be deleted
+    * array("attributename" => array("value1", "value2") - The given values
+    *                                                      will be deleted
+    * If $attr is null or omitted , then the whole Entry will be deleted!
+    *
+    * These changes are local to the entry and do
+    * not affect the entry on the server until {@link update()} is called.
+    *
+    * Please note that you must select the attribute (at $ldap->search() for example)
+    * to be able to delete values of it, Otherwise {@link update()} will silently fail
+    * and remove nothing.
+    *
+    * @param string|array $attr Attributes to delete (NULL or missing to delete whole entry)
+    *
+    * @access public
+    * @return true
+    */
     function delete($attr = null)
     {
         if (is_null($attr)) {
             $this->_delete = true;
-            return;
+            return true;
         }
         if (is_string($attr)) {
             $attr = array($attr);
@@ -472,23 +503,27 @@ class Net_LDAP_Entry extends PEAR
                 }
             }
         }
+        $return = true;
+        return $return;
     }
 
     /**
-     * Replaces attributes or its values
-     *
-     * The parameter has to an array of the following form:
-     * array("attributename" => "single value",
-     *       "attribute2name" => array("value1", "value2"))
-     * If the attribute does not yet exist it will be added instead.
-     * If the attribue value is null, the attribute will de deleted
-     *
-     * These changes are local to the entry and do
-     * not affect the entry on the server until {@link update()} is called.
-     *
-     * @access public
-     * @param array $attr
-     */
+    * Replaces attributes or its values
+    *
+    * The parameter has to an array of the following form:
+    * array("attributename" => "single value",
+    *       "attribute2name" => array("value1", "value2"))
+    * If the attribute does not yet exist it will be added instead.
+    * If the attribue value is null, the attribute will de deleted
+    *
+    * These changes are local to the entry and do
+    * not affect the entry on the server until {@link update()} is called.
+    *
+    * @param array $attr Attributes to replace
+    *
+    * @access public
+    * @return true|Net_LDAP_Error
+    */
     function replace($attr = array())
     {
         if (false == is_array($attr)) {
@@ -508,73 +543,101 @@ class Net_LDAP_Entry extends PEAR
             // existing attributes will get replaced
             if ($this->exists($k)) {
                 $this->_changes["replace"][$k] = $v;
-                $this->_attributes[$k] = $v;
+                $this->_attributes[$k]         = $v;
             } else {
                 // new ones just get added
                 $this->add(array($k => $v));
             }
         }
+        $return = true;
+        return $return;
     }
 
     /**
-     * Update the entry on the directory server
-     *
-     * @access public
-     * @param Net_LDAP $ldap (optional) If you provide a Net_LDAP object, be sure to PASS IT VIA REFERENCE!
-     * @return mixed
-     */
-    function update($ldap=false)
+    * Update the entry on the directory server
+    *
+    * @param Net_LDAP $ldap If passed, a call to setLDAP() is issued prior update, thus switching the LDAP-server. This is for perl-ldap interface compliance
+    *
+    * @access public
+    * @return true|Net_LDAP_Error
+    * @todo Entry rename with a DN containing special characters needs testing!
+    */
+    function update($ldap = null)
     {
-        if (!$ldap) {  // If object is not provided, then use this entrys ldap object
-            $ldap =& $this->_ldap;
-        } else {
-            if (!is_a($ldap, 'Net_LDAP')) {
-                return PEAR::raiseError("Need a Net_LDAP object as parameter");
+        if ($ldap) {
+            $msg = $this->setLDAP($ldap);
+            if (Net_LDAP::isError($msg)) {
+                return PEAR::raiseError('You passed an invalid $ldap variable to update()');
             }
         }
 
+        // ensure we have a valid LDAP object
+        $ldap =& $this->getLDAP();
+        if (!is_a($ldap, 'Net_LDAP')) {
+            return PEAR::raiseError("The entries LDAP object is not valid");
+        }
+
         $link = $ldap->getLink();
 
-        // Delete the entry
-        if ($this->_delete === true) {
+        /*
+        * Delete the entry
+        */
+        if (true === $this->_delete) {
             return $ldap->delete($this);
         }
-        // New entry
-        if ($this->_new === true) {
+
+        /*
+        * New entry
+        */
+        if (true === $this->_new) {
             $msg = $ldap->add($this);
             if (Net_LDAP::isError($msg)) {
                 return $msg;
             }
-            $this->_new = false;
-            $this->_changes['add'] = array();
-            $this->_changes['delete'] = array();
+            $this->_new                = false;
+            $this->_changes['add']     = array();
+            $this->_changes['delete']  = array();
             $this->_changes['replace'] = array();
-            $this->_original = $this->_attributes;
-            return;
+            $this->_original           = $this->_attributes;
+
+            $return = true;
+            return $return;
         }
-        // Rename/move entry
+
+        /*
+        * Rename/move entry
+        */
         if (false == is_null($this->_newdn)) {
             if ($ldap->getLDAPVersion() !== 3) {
                 return PEAR::raiseError("Renaming/Moving an entry is only supported in LDAPv3");
             }
             // make dn relative to parent (needed for ldap rename)
-            $parent = Net_LDAP_Util::ldap_explode_dn_escaped($this->_newdn, 0);
-            //NEW CODE: $parent = Net_LDAP_Util::ldap_explode_dn($this->_newdn, array('casefolding' => 'none', 'reverse' => false, 'onlyvalues' => false))
+            $parent = Net_LDAP_Util::ldap_explode_dn($this->_newdn, array('casefolding' => 'none', 'reverse' => false, 'onlyvalues' => false));
             if (Net_LDAP::isError($parent)) {
                 return $parent;
             }
             $child = array_shift($parent);
-            $parent = join(",", $parent);
+            // maybe the dn consist of a multivalued RDN, we must build the dn in this case
+            // because the $child-RDN is an array!
+            if (is_array($child)) {
+                $child = Net_LDAP_Util::canonical_dn($child);
+            }
+            $parent = Net_LDAP_Util::canonical_dn($parent);
+
             // rename
             if (false == @ldap_rename($link, $this->_dn, $child, $parent, true)) {
                 return PEAR::raiseError("Entry not renamed: " .
                                         @ldap_error($link), @ldap_errno($link));
             }
             // reflect changes to local copy
-            $this->_dn = $this->_newdn;
+            $this->_dn    = $this->_newdn;
             $this->_newdn = null;
         }
-        // Modified entry
+
+        /*
+        * Carry out modifications to the entry
+        */
+        // ADD
         foreach ($this->_changes["add"] as $attr => $value) {
             // if attribute exists, add new values
             if ($this->exists($attr)) {
@@ -591,8 +654,9 @@ class Net_LDAP_Entry extends PEAR
             }
             // all went well here, I guess
             unset($this->_changes["add"][$attr]);
-        } // add
+        }
 
+        // DELETE
         foreach ($this->_changes["delete"] as $attr => $value) {
             // In LDAPv3 you need to specify the old values for deleting
             if (is_null($value) && $ldap->getLDAPVersion() === 3) {
@@ -603,27 +667,32 @@ class Net_LDAP_Entry extends PEAR
                                         @ldap_error($link), @ldap_errno($link));
             }
             unset($this->_changes["delete"][$attr]);
-        } // delete
+        }
 
+        // REPLACE
         foreach ($this->_changes["replace"] as $attr => $value) {
             if (false === @ldap_modify($link, $this->dn(), array($attr => $value))) {
                 return PEAR::raiseError("Could not replace attribute $attr values: " .
                                         @ldap_error($link), @ldap_errno($link));
             }
             unset($this->_changes["replace"][$attr]);
-        } // replace
+        }
 
         // all went well, so _original (server) becomes _attributes (local copy)
         $this->_original = $this->_attributes;
+
+        $return = true;
+        return $return;
     }
 
     /**
-     * Returns the right attribute name
-     *
-     * @access private
-     * @param string $attr Name of attribute
-     * @return string The right name of the attribute
-     */
+    * Returns the right attribute name
+    *
+    * @param string $attr Name of attribute
+    *
+    * @access private
+    * @return string The right name of the attribute
+    */
     function _getAttrName($attr)
     {
         $name = strtolower($attr);
@@ -634,49 +703,11 @@ class Net_LDAP_Entry extends PEAR
     }
 
     /**
-     * Copy the current entry to another place in the directory
-     *
-     * @access public
-     * @param Net_LDAP $ldap       Net_LDAP
-     * @param string   $dn         New distinguished name
-     * @param boolean  $relative   Is the new name relative to current parent
-     * @return Net_LDAP_Entry|Net_LDAP_Error   Reference to the copied Net_LDAP_Entry or Net_LDAP_Error
-     */
-    function &copy(&$ldap, $dn, $relative = false)
-    {
-        if ($relative == true) {
-            $dn = "$dn," . $this->dn();
-        }
-        // get the attribute which makes up the rdn
-        $parts = @ldap_explode_dn($this->dn(), 0);
-        list($attr, $value) = explode('=', $parts[0]);
-
-        // remove it from the entry (not valid in copy)
-        $old_e = $this; // backup
-        $old_e->delete(array($attr => $value));
-
-        // get the attribute which makes up the new rdn
-        $parts = @ldap_explode_dn($dn, 0);
-        list($attr, $value) = explode('=', $parts[0]);
-
-        $old_e->add(array($attr => $value));
-
-        $entry = new Net_LDAP_Entry(null, $dn);
-
-        $entry->add($old_e->getValues());
-        $msg = $entry->update($ldap);
-        if (Net_LDAP::isError($msg)) {
-            return $msg;
-        }
-        return $entry;
-    }
-
-    /**
-     * Returns a reference to the LDAP-Object of this entry
-     *
-     * @access public
-     * @return Net_LDAP|Net_LDAP_Error   Reference to the Net_LDAP Object (the connection) or Net_LDAP_Error
-     */
+    * Returns a reference to the LDAP-Object of this entry
+    *
+    * @access public
+    * @return Net_LDAP|Net_LDAP_Error   Reference to the Net_LDAP Object (the connection) or Net_LDAP_Error
+    */
     function &getLDAP()
     {
         if (!is_a($this->_ldap, 'Net_LDAP')) {
@@ -686,6 +717,27 @@ class Net_LDAP_Entry extends PEAR
         }
     }
 
+    /**
+    * Sets a reference to the LDAP-Object of this entry
+    *
+    * After setting a Net_LDAP object, calling update() will use that object for
+    * updating directory contents. Use this to dynamicly switch directorys.
+    *
+    * @param Net_LDAP &$ldap Net_LDAP object that this entry should be connected to
+    *
+    * @access public
+    * @return true|Net_LDAP_Error
+    */
+    function setLDAP(&$ldap)
+    {
+        if (!is_a($ldap, 'Net_LDAP')) {
+            return PEAR::raiseError("LDAP is not a valid Net_LDAP object");
+        } else {
+            $this->_ldap =& $ldap;
+            return true;
+        }
+    }
+
     /**
     * Applies a regular expression onto a single- or multivalued attribute (like preg_match())
     *
@@ -724,9 +776,11 @@ class Net_LDAP_Entry extends PEAR
     * @param string $regex     The regular expression
     * @param string $attr_name The attribute to search in
     * @param array  $matches   (optional, PASS BY REFERENCE!) Array to store matches in
+    *
     * @return boolean|Net_LDAP_Error  TRUE, if we had a match in one of the values, otherwise false. Net_LDAP_Error in case something went wrong
     */
-    function preg_match($regex, $attr_name, $matches = array()){
+    function preg_match($regex, $attr_name, $matches = array())
+    {
         $matches = array();
 
         // fetch attribute values
